/***************************************************************************
*   Copyright (C) 2004 by Hideki Ikemoto , (c) 2004 by 421                *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

/* Kita Image Viewer */

#include "kitaimgtabwidget.h"
#include "libkita/signalcollection.h"

#include <klibloader.h>
#include <kapplication.h>
#include <klocale.h>
#include <kstdaccel.h>
#include <kpopupmenu.h>
#include <kparts/browserextension.h>
#include <kparts/partmanager.h>
#include <kio/netaccess.h>
#include <kaction.h>
#include <krun.h>

#include <qlayout.h>
#include <qiconset.h>
#include <qclipboard.h>
#include <qmessagebox.h>


/*--------------------------------------------------------------*/


KitaImgTabWidget::KitaImgTabWidget( QWidget* parent, const char* name, WFlags fl )
    :KitaTabWidgetBase( parent, name, fl )
{
    setXMLFile( "kitaimgtabwidgetui.rc" );
    setTabBar( new KitaImgTabBar( this ) );
    setupActions();
}


KitaImgTabWidget::~KitaImgTabWidget(){}


/* public slot */
void KitaImgTabWidget::slotOpenURL( const KURL& url )
{
    KitaImgView* view = findImgView( url );
    if( view ){
	setCurrentPage( indexOf( view ) );
	return;
    }

    /* create and show new image viewer */
    view = new KitaImgView( url, this, "Viewer", 0L );
    if( view ){
	addTab( view, QIconSet( view->icon(), QIconSet::Small ), "" );
	view->slotShowImage();
	showPage( view );

	/* resize */
	QRect rt = view->rect();
	if( rt.right()-rt.left() < view->imgWidth()
	    || rt.bottom()-rt.top() < view->imgHeight() ) view->slotFitToWin();
    }
}


/* protected */ /* virtual */
void KitaImgTabWidget::customEvent( QCustomEvent * e )
{
    KitaTabWidgetBase::customEvent( e );
    if ( e->type() == EVENT_FitImageToWinEvent ) slotShowFitToWin();
}


/* protected */ /* virtual */
void KitaImgTabWidget::deleteWidget( QWidget* w )
{
    KitaTabWidgetBase::deleteWidget( w );

    if ( count() == 0 ) {
	static_cast< KitaDockWidgetBase* >( parentWidget() )->slotHideDock();
    }  
}


/* private */
KitaImgView* KitaImgTabWidget::findImgView( const KURL& url )
{
    int max = count();
    if( max == 0 ) return NULL;
    int i = 0;

    while( i < max ){
	KitaImgView* view = isImgView( page ( i ) );
	if( view ){
	    if ( view->url() == url ) return view;
	}
	i++;
    }

    return NULL;
}


/* private */
KitaImgView* KitaImgTabWidget::isImgView( QWidget* w )
{
    KitaImgView* view = NULL;
    if( w ){
	if( w->isA( "KitaImgView" ) ) view = static_cast< KitaImgView* >( w );
    }

    return view;
}



/*------------------------------------*/
/* Image View actions */


/* private */
void KitaImgTabWidget::setupActions(){

    new KAction( i18n( "Cancel mosaic" ),
                 Key_X,
                 this,
                 SLOT( slotCancelMosaic() ),
                 actionCollection(),
                 "imgviewer_cancelmosaic" );

    new KAction( i18n( "Fit Image to Window" ),
                 0,
                 this,
                 SLOT( slotShowFitToWin() ),
                 actionCollection(),
                 "imgviewer_fittowin" );

    new KAction( i18n( "Original Size" ),
                 0,
                 this,
                 SLOT( slotOriginalSize() ),
                 actionCollection(),
                 "imgviewer_original" );

    new KAction( i18n( "copy URL" ),
                 0,
                 this,
		 SLOT( slotCopyURL() ),
                 actionCollection(),
                 "imgviewer_copyurl" );   

    new KAction( i18n( "Open with Web Browser" ),
                 0,
                 this,
                 SLOT( slotOpenBrowser() ),
                 actionCollection(),
                 "imgviewer_openbrowser" );
}


/* public slot */
void KitaImgTabWidget::slotShowFitToWin()
{
    KitaImgView* view = isImgView( currentPage() );
    if( view ) view->slotFitToWin();
}


/* public slot */
void KitaImgTabWidget::slotOriginalSize()
{
    KitaImgView* view = isImgView( currentPage() );
    if( view ) view->slotOriginalSize();
}


/* public slot */
void KitaImgTabWidget::slotCancelMosaic()
{
    KitaImgView* view = isImgView( currentPage() );
    if( view && view->mosaic() ){
	QRect rt = view->rect();
	int wd = view->imgWidth();
	int ht = view->imgHeight();
	view->slotCancelMosaic();

	/* resize */
	if( rt.right()-rt.left() < wd
	    || rt.bottom()-rt.top() < ht ){
	
	    /* The image is not shown here, and kview can't get the exact image size.
	       So, post custom event, and fit image size after showing it. */
	    FitImageToWinEvent* e = new FitImageToWinEvent();
	    QApplication::postEvent( this, e );  // Qt will delete it when done
	}
    }
}



/* public slot */
void KitaImgTabWidget::slotCopyURL()
{
    QClipboard* clipboard = QApplication::clipboard();
    
    KitaImgView* view = isImgView( currentPage() );
    if( view ){
	QString cliptxt = view->url().prettyURL();
    
	clipboard->setText( cliptxt , QClipboard::Clipboard );
	clipboard->setText( cliptxt , QClipboard::Selection );
    }
}



/* public slot */ 
void KitaImgTabWidget::slotOpenBrowser(){

    KitaImgView* view = isImgView( currentPage() );
    if( view ) KRun::runURL( view->url(), "text/html" );
}




/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/



KitaImgTabBar::KitaImgTabBar( QWidget* parent, const char* name ) : KitaTabBarBase( parent, name ){}

KitaImgTabBar::~KitaImgTabBar(){}

/* popup menu */ /* protected */ /* virtual */
void KitaImgTabBar::contextMenuEvent( QContextMenuEvent* e )
{
    KPopupMenu* popup = new KPopupMenu( this );
    KActionCollection * collection = static_cast<KitaImgTabWidget*>( parentWidget() )->actionCollection();
    plugDefaultActions( popup );

    popup->insertSeparator();
    collection->action( "imgviewer_cancelmosaic" )->plug( popup );
    collection->action( "imgviewer_fittowin" )->plug( popup );
    collection->action( "imgviewer_original" )->plug( popup );
    collection->action( "imgviewer_openbrowser" )->plug( popup );
    collection->action( "imgviewer_copyurl" )->plug( popup );
    
    popup->insertSeparator();
    collection->action( "tab_configkeys" )->plug( popup );       

    
    popup->exec( e->globalPos() );
    delete popup;
}



/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/



KitaImgDock::KitaImgDock( KDockManager* dockManager,
		    const char* name,
		    const QPixmap &pixmap,
		    QWidget* parent,
		    const QString& strCaption,
		    const QString& strTabPageLabel,
		    WFlags f )
    :KitaDockWidgetBase( dockManager, name, pixmap, parent, strCaption, strTabPageLabel, f ){}


KitaImgDock::~KitaImgDock(){}


/* setup & create KitaImgTabWidget */ /* public */
KitaImgTabWidget* KitaImgDock::setup()
{
    /* create KitaNaviTabWidget */
    m_imgTab = new KitaImgTabWidget( this, "Image Viewer" );
    setWidget( m_imgTab );

    Kita::SignalCollection* signalCollection = Kita::SignalCollection::getInstance();
    connect( signalCollection, SIGNAL( switchToImgview() ),
	     SLOT( slotShowDock() ) );
    
    return m_imgTab;
}


/* public slot */
void KitaImgDock::slotOpenURL( const KURL& url )
{
    slotShowDock();
    
    if( m_imgTab ) m_imgTab->slotOpenURL( url );
}

    

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Image Viewer Widget */
/* KViewViewer is embedded in the viewer */

KitaImgView::KitaImgView( const KURL& url,
			      QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    m_url = url;
    m_part = NULL;
    m_mosaic = TRUE;
    m_layout = new QVBoxLayout( this, 0, 0, "BoxLayout");
    m_wd = 0;
    m_ht = 0;

    /* download image file */    
    QString tmpFile;
    if ( KIO::NetAccess::download( m_url, tmpFile ) ){

	m_orgimg = QImage( tmpFile );

	if( !m_orgimg.isNull() ){

	    m_wd = m_orgimg.width();
	    m_ht = m_orgimg.height();

	    /* create icon */
	    m_icon.convertFromImage( m_orgimg.scale( 32, 32, QImage::ScaleMin ) );
	}
	
	KIO::NetAccess::removeTempFile( tmpFile );
    }	    
}


KitaImgView::~KitaImgView()
{
    if( m_part ) delete m_part->widget();
    if( m_layout ) delete m_layout;
}


/* public */
const KURL KitaImgView::url() const
{
    return m_url;
}


/* public */
const QPixmap& KitaImgView::icon() {
    return m_icon;
}


/* public */
const bool KitaImgView::mosaic(){
    return m_mosaic;
}


/* public */
const int KitaImgView::imgWidth(){
    return m_wd;
}


/* public */
const int KitaImgView::imgHeight(){
    return m_ht;
}


/* public slot */
void KitaImgView::slotShowImage()
{
    if( m_orgimg.isNull() ) return;

    createKView();

    if( m_part ){
	if( m_mosaic ){
	    QImage img = m_orgimg.scale( 48, 48, QImage::ScaleMin).scale( m_wd, m_ht, QImage::ScaleMin );
	    emit newImage( img );
	}
	else{
	    emit newImage( m_orgimg );
	    m_orgimg = QImage(); /* reset */
	}
    }
}


/* public slot */
void KitaImgView::slotCancelMosaic()
{
    if( !m_mosaic ) return;
    m_mosaic = FALSE;
    slotShowImage();
}


/* public slot */
void KitaImgView::slotFitToWin()
{
    if( m_part ) m_part->action( "fittowin" )->activate();
}


/* public slot */
void KitaImgView::slotOriginalSize()
{
    if( m_part ){
	KSelectAction* act = static_cast< KSelectAction* >( m_part->action( "view_zoom" ));
	QStringList items = act->items();

	int i = 0;
	QStringList::iterator it = items.begin();
	for ( ; it != items.end(); ++it, ++i ){
	    if( (*it) == "100%" ) break;
	}
	act->setCurrentItem( i );
	act->activate();
    }
}



/* private slot */
void KitaImgView::slotPopupMenu( KXMLGUIClient* , const QPoint& global, const KURL& , const QString& , mode_t  )
{
    if( !m_part ) return;
    
    KPopupMenu *popup = new KPopupMenu( this );

    m_part->action( "fittowin" )->plug( popup ); 
    m_part->action( "view_zoom" )->plug( popup );
    m_part->action( "zoomin" )->plug( popup ); 
    m_part->action( "zoomout" )->plug( popup );
    if( m_part->action( "rotateCW" ) )  m_part->action( "rotateCW" )->plug( popup );
    if( m_part->action( "rotateCCW" ) ) m_part->action( "rotateCCW" )->plug( popup );
    
    popup->exec( global );
    delete popup;
}


/* create kviewviewer and connect signals */ /* private */
void KitaImgView::createKView()
{
    /* remove the previous part */

    /* Emitting newImage signal without removing the previous part
       always opens the "save as" dialog box.
       So, to avoid it, you need to remove the previous part here. */
    if( m_part ) delete m_part->widget();
    m_part = NULL;
    
    /* create part */
    KLibrary* lib = KLibLoader::self()->library( "libkviewviewer" );
    KLibFactory* factory;
    if( lib ) factory = lib->factory();
    if( ! lib || ! factory ){
       QMessageBox::critical( parentWidget(), i18n( " Load Error" ),
			      QString( i18n( "can't load libkviewviewer.") ) );
        return;
    }
    
    m_part = static_cast<KParts::ReadWritePart *>
	 ( factory->create( this, 0L, "KParts::ReadWritePart" ) );

    if( !m_part ){
       QMessageBox::critical( parentWidget(), i18n( " Load Error" ),
			      QString( i18n( "can't create KViewViewer.") ) );
        return;	
    }
    
    m_layout->addWidget( m_part->widget() );

    connect( this, SIGNAL( newImage( const QImage & ) ),
	     m_part, SLOT( newImage( const QImage & ) ));	    

    KParts::BrowserExtension* ext = KParts::BrowserExtension::childObject( m_part );
    if( ext ){
	connect( ext, SIGNAL( popupMenu( KXMLGUIClient*, const QPoint&,
					 const KURL&, const QString&, mode_t ) ),
		 SLOT( slotPopupMenu( KXMLGUIClient* , const QPoint& , const KURL& , const QString& , mode_t ) ) );
    }

    m_part->widget()->show();
}
